@@ -2,10 +2,10 @@
 
    Copyright (C) 2006 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Written by Paul Eggert <eggert@cs.ucla.edu>.  */
 
@@ -53,9 +52,9 @@ xmemxfrm (char *restrict dest, size_t destsize,
   if (errno)
     {
       error (0, errno, _("string transformation failed"));
-      error (0, 0, _("Set LC_ALL='C' to work around the problem."));
+      error (0, 0, _("set LC_ALL='C' to work around the problem"));
       error (exit_failure, 0,
-	     _("The untransformed string was %s."),
+	     _("the untransformed string was %s"),
 	     quotearg_n_style_mem (0, locale_quoting_style, src, srcsize));
     }
 
