@@ -1,5 +1,5 @@
 /* cp-hash.c  -- file copying (hash search routines)
-   Copyright (C) 89, 90, 91, 1995-2005 Free Software Foundation.
+   Copyright (C) 89, 90, 91, 1995-2008 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,14 +19,11 @@
 
 #include <config.h>
 
-#include <stdio.h>
 #include <sys/types.h>
 #include "system.h"
 
 #include "same.h"
-#include "quote.h"
 #include "hash.h"
-#include "error.h"
 #include "cp-hash.h"
 
 /* Use ST_DEV and ST_INO as the key, FILENAME as the value.
@@ -97,24 +94,6 @@ forget_created (ino_t ino, dev_t dev)
     src_to_dest_free (ent);
 }
 
-/* Add FILE to the list of files that we have created.
-   Return true if successful.  */
-
-extern bool
-remember_created (char const *file)
-{
-  struct stat sb;
-
-  if (stat (file, &sb) < 0)
-    {
-      error (0, errno, "%s", quote (file));
-      return false;
-    }
-
-  remember_copied (file, sb.st_ino, sb.st_dev);
-  return true;
-}
-
 /* If INO/DEV correspond to an already-copied source file, return the
    name of the corresponding destination file.  Otherwise, return NULL.  */
 
