@@ -24,7 +24,6 @@
 
 #include "system.h"
 #include "error.h"
-#include "lchmod.h"
 #include "mkdir-p.h"
 #include "modechange.h"
 #include "prog-fprintf.h"
@@ -34,10 +33,7 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "mkdir"
 
-#define AUTHORS "David MacKenzie"
-
-/* The name this program was run with. */
-char *program_name;
+#define AUTHORS proper_name ("David MacKenzie")
 
 static struct option const longopts[] =
 {
@@ -58,7 +54,7 @@ usage (int status)
 	     program_name);
   else
     {
-      printf (_("Usage: %s [OPTION] DIRECTORY...\n"), program_name);
+      printf (_("Usage: %s [OPTION]... DIRECTORY...\n"), program_name);
       fputs (_("\
 Create the DIRECTORY(ies), if they do not already exist.\n\
 \n\
@@ -153,7 +149,7 @@ main (int argc, char **argv)
   options.created_directory_format = NULL;
 
   initialize_main (&argc, &argv);
-  program_name = argv[0];
+  set_program_name (argv[0]);
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
