@@ -73,7 +73,7 @@
 
    The interference of the POSIX-compliant small letter options -w and -s:
    (`interference' means `setting a _separator_ with -s switches off the
-   column sturctur and the default - not generally - page_width,
+   column structure and the default - not generally - page_width,
    acts on -w option')
        options:       text form  / separator:     equivalent new options:
        -w l   -s[x]
@@ -92,7 +92,7 @@
    Options:
 
    Including version 1.22i:
-   Some SMALL LETTER options has been redefined with the object of a
+   Some SMALL LETTER options have been redefined with the object of a
    better POSIX compliance. The output of some further cases has been
    adapted to other UNIXes. A violation of downward compatibility has to
    be accepted.
@@ -413,8 +413,6 @@ struct COLUMN
 
 typedef struct COLUMN COLUMN;
 
-#define NULLCOL (COLUMN *)0
-
 static int char_to_clump (char c);
 static bool read_line (COLUMN *p);
 static bool print_page (void);
@@ -2404,8 +2402,10 @@ print_header (void)
   lhs_spaces = available_width >> 1;
   rhs_spaces = available_width - lhs_spaces;
 
-  printf ("\n\n%s%*s%s%*s%s\n\n\n",
-	  date_text, lhs_spaces, " ", file_text, rhs_spaces, " ", page_text);
+  printf ("\n\n%*.*s%s%*.*s%s%*.*s%s\n\n\n",
+          chars_per_margin, chars_per_margin, " ",
+          date_text, lhs_spaces, lhs_spaces, " ",
+          file_text, rhs_spaces, rhs_spaces, " ", page_text);
 
   print_a_header = false;
   output_position = 0;
